import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { DistributeTransactionDto } from './dto/distribute-transaction.dto';
import { OrderTransactionDto } from './dto/order-transaction.dto';
import { InjectRepository } from '@nestjs/typeorm';
import { Transactions } from './entities/transactions.entity';
import { Connection, Repository } from 'typeorm';
import { COA } from './entities/coa.entity';
import { TransactionType } from './entities/transaction-type.entity';
import { TransactionJournal } from './entities/transaction-journal.entity';
import { CoaService } from './coa.service';
import {
  balanceType,
  coaType,
  statusTransaction,
  typeTransaction,
} from '../helper/enum-list';
import { ProductService } from '../product/product.service';
import * as irsService from '../helper/irs-service';

@Injectable()
export class TransactionService {
  constructor(
    @InjectRepository(Transactions)
    private transactionRepository: Repository<Transactions>,
    @InjectRepository(TransactionType)
    private transactionTypeRepository: Repository<TransactionType>,
    @InjectRepository(TransactionJournal)
    private transactionJournalRepository: Repository<TransactionJournal>,
    @InjectRepository(COA)
    private coaRepository: Repository<COA>,
    private coaService: CoaService,
    private productService: ProductService,
    private connection: Connection,
  ) {}

  async distributeDeposit(distributeTransactionDto: DistributeTransactionDto) {
    // GET COA
    const coaSender = await this.coaService.findByUser(
      'id_user',
      coaType.WALLET,
    );
    const coaReciever = await this.coaService.findByUser(
      'id_user',
      coaType.WALLET,
    );

    await this.connection.transaction(async (manager) => {
      //INSERT TRANSACTION
      const transactionSaved = await manager.insert(Transactions, {
        amount: distributeTransactionDto.amount,
        user: 'id_user',
        userDestination: distributeTransactionDto.destination,
        status: statusTransaction.SUCCESS,
        type: typeTransaction.DISTRIBUTION,
      });

      //INSERT TRANSACTION JOURNAL FOR SENDER
      const journalSender = await manager.insert(TransactionJournal, {
        amount: distributeTransactionDto.amount,
        transaction: transactionSaved.identifiers[0],
        coa: coaSender,
        type: balanceType.CREDIT,
      });

      //INSERT TRANSACTION JOURNAL FOR RECEIVER
      const journalReceiver = await manager.insert(TransactionJournal, {
        amount: distributeTransactionDto.amount,
        transaction: transactionSaved.identifiers[0],
        coa: coaReciever,
        type: balanceType.DEBIT,
      });

      //UPDATE AMOUNT COA SENDER
      coaSender.amount = coaSender.amount - distributeTransactionDto.amount;
      await manager.save(coaSender);

      coaReciever.amount = coaReciever.amount + distributeTransactionDto.amount;
      await manager.save(coaReciever);
    });

    return true;
  }

  async orderTransaction(orderTransactionDto: OrderTransactionDto) {
    const coaAccount = await this.coaService.findByUser(
      'id_user',
      coaType.WALLET,
    );

    //GET PRODUCT
    const product = await this.productService.findOne(
      orderTransactionDto.productCode,
    );

    if (coaAccount.amount <= product.price) {
      throw new HttpException(
        {
          statusCode: HttpStatus.INTERNAL_SERVER_ERROR,
          error: `Transaction Failed because saldo not enough`,
        },
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }

    try {
      const orderIRS = await irsService.createTransaction(
        orderTransactionDto.productCode,
        orderTransactionDto.destination,
      );

      if (orderIRS.success) {
        await this.connection.transaction(async (manager) => {
          //INSERT TRANSACTION
          const transactionSaved = await manager.insert(Transactions, {
            amount: product.price,
            user: 'id_user',
            status: statusTransaction.SUCCESS,
            type: typeTransaction.ORDER,
          });

          //INSERT TRANSACTION JOURNAL
          const journalSender = await manager.insert(TransactionJournal, {
            amount: product.price,
            transaction: transactionSaved.identifiers[0],
            coa: coaAccount,
            type: balanceType.CREDIT,
          });

          //UPDATE AMOUNT COA
          coaAccount.amount = coaAccount.amount - product.price;
          await manager.save(coaAccount);
        });
      } else {
        throw new HttpException(
          {
            statusCode: HttpStatus.INTERNAL_SERVER_ERROR,
            error: `Transaction Failed because ${orderIRS.msg}`,
          },
          HttpStatus.INTERNAL_SERVER_ERROR,
        );
      }
    } catch (e) {
      throw e;
    }

    return true;
  }
}
